@@ -604,21 +604,21 @@ def share_snapshot_get(context, snapshot_id):
     return IMPL.share_snapshot_get(context, snapshot_id)
 
 
-def share_snapshot_get_all(context, filters=None, sort_key=None,
-                           sort_dir=None):
+def share_snapshot_get_all(context, filters=None, limit=None, offset=None,
+                           sort_key=None, sort_dir=None):
     """Get all snapshots."""
     return IMPL.share_snapshot_get_all(
-        context, filters=filters, sort_key=sort_key, sort_dir=sort_dir,
-    )
+        context, filters=filters, limit=limit, offset=offset,
+        sort_key=sort_key, sort_dir=sort_dir)
 
 
 def share_snapshot_get_all_by_project(context, project_id, filters=None,
-                                      sort_key=None, sort_dir=None):
+                                      limit=None, offset=None, sort_key=None,
+                                      sort_dir=None):
     """Get all snapshots belonging to a project."""
     return IMPL.share_snapshot_get_all_by_project(
-        context, project_id, filters=filters, sort_key=sort_key,
-        sort_dir=sort_dir,
-    )
+        context, project_id, filters=filters, limit=limit, offset=offset,
+        sort_key=sort_key, sort_dir=sort_dir)
 
 
 def share_snapshot_get_all_for_share(context, share_id, filters=None,
@@ -766,13 +766,16 @@ def security_service_get_all_by_project(context, project_id):
 
 
 ####################
-
-
 def share_metadata_get(context, share_id):
     """Get all metadata for a share."""
     return IMPL.share_metadata_get(context, share_id)
 
 
+def share_metadata_get_item(context, share_id, key):
+    """Get metadata item for given key and for a given share.."""
+    return IMPL.share_metadata_get_item(context, share_id, key)
+
+
 def share_metadata_delete(context, share_id, key):
     """Delete the given metadata item."""
     IMPL.share_metadata_delete(context, share_id, key)
@@ -783,6 +786,11 @@ def share_metadata_update(context, share, metadata, delete):
     IMPL.share_metadata_update(context, share, metadata, delete)
 
 
+def share_metadata_update_item(context, share_id, item):
+    """update meta item containing key and value for given share."""
+    IMPL.share_metadata_update_item(context, share_id, item)
+
+
 ###################
 
 def share_export_location_get_by_uuid(context, export_location_uuid,
@@ -868,6 +876,11 @@ def share_network_get(context, id):
     return IMPL.share_network_get(context, id)
 
 
+def share_network_get_all_by_filter(context, filters=None):
+    """Get all share network DB records for the given filter."""
+    return IMPL.share_network_get_all_by_filter(context, filters=filters)
+
+
 def share_network_get_all(context):
     """Get all share network DB records."""
     return IMPL.share_network_get_all(context)
@@ -1070,6 +1083,11 @@ def share_server_backend_details_set(context, share_server_id, server_details):
                                                  server_details)
 
 
+def share_server_backend_details_delete(context, share_server_id):
+    """Delete backend details DB records for a share server."""
+    return IMPL.share_server_backend_details_delete(context, share_server_id)
+
+
 def share_servers_update(context, share_server_ids, values):
     """Updates values of a bunch of share servers at once."""
     return IMPL.share_servers_update(
