@@ -35,6 +35,7 @@ class NetAppCmodeSingleSvmShareDriver(driver.ShareDriver):
             False, *args, **kwargs)
         self.library = lib_single_svm.NetAppCmodeSingleSVMFileStorageLibrary(
             self.DRIVER_NAME, **kwargs)
+        self.dhss_mandatory_security_service_association = {}
 
     def do_setup(self, context):
         self.library.do_setup(context)
@@ -104,13 +105,13 @@ class NetAppCmodeSingleSvmShareDriver(driver.ShareDriver):
 
     def _update_share_stats(self, data=None):
         data = self.library.get_share_stats(
-            filter_function=self.get_filter_function(),
+            get_filter_function=self.get_filter_function,
             goodness_function=self.get_goodness_function())
         super(NetAppCmodeSingleSvmShareDriver, self)._update_share_stats(
             data=data)
 
-    def get_default_filter_function(self):
-        return self.library.get_default_filter_function()
+    def get_default_filter_function(self, pool=None):
+        return self.library.get_default_filter_function(pool=pool)
 
     def get_default_goodness_function(self):
         return self.library.get_default_goodness_function()
@@ -280,3 +281,54 @@ class NetAppCmodeSingleSvmShareDriver(driver.ShareDriver):
 
     def unmanage_server(self, server_details, security_services=None):
         raise NotImplementedError
+
+    def get_share_status(self, share_instance, share_server=None):
+        return self.library.get_share_status(share_instance, share_server)
+
+    def share_server_migration_start(self, context, src_share_server,
+                                     dest_share_server, shares, snapshots):
+        raise NotImplementedError
+
+    def share_server_migration_continue(self, context, src_share_server,
+                                        dest_share_server, shares, snapshots):
+        raise NotImplementedError
+
+    def share_server_migration_complete(self, context, src_share_server,
+                                        dest_share_server, shares, snapshots,
+                                        new_network_info):
+        raise NotImplementedError
+
+    def share_server_migration_cancel(self, context, src_share_server,
+                                      dest_share_server, shares, snapshots):
+        raise NotImplementedError
+
+    def share_server_migration_check_compatibility(
+            self, context, share_server, dest_host, old_share_network,
+            new_share_network, shares_request_spec):
+        raise NotImplementedError
+
+    def share_server_migration_get_progress(self, context, src_share_server,
+                                            dest_share_server):
+        raise NotImplementedError
+
+    def choose_share_server_compatible_with_share(self, context, share_servers,
+                                                  share, snapshot=None,
+                                                  share_group=None):
+        raise NotImplementedError
+
+    def choose_share_server_compatible_with_share_group(
+            self, context, share_servers, share_group_ref,
+            share_group_snapshot=None):
+        raise NotImplementedError
+
+    def update_share_server_security_service(
+            self, context, share_server, network_info, share_instances,
+            share_instance_rules, new_security_service,
+            current_security_service=None):
+        raise NotImplementedError
+
+    def check_update_share_server_security_service(
+            self, context, share_server, network_info, share_instances,
+            share_instance_rules, new_security_service,
+            current_security_service=None):
+        raise NotImplementedError
