@@ -97,7 +97,7 @@ class TegileAPIExecutor(object):
         return self._send_api_request(*args, **kwargs)
 
     @debugger
-    @utils.retry(exception=(requests.ConnectionError, requests.Timeout),
+    @utils.retry(retry_param=(requests.ConnectionError, requests.Timeout),
                  interval=30,
                  retries=3,
                  backoff_rate=1)
@@ -447,6 +447,9 @@ class TegileShareDriver(driver.ShareDriver):
                 pool['qos'] = pool.pop('QoS_support', False)
                 pool['reserved_percentage'] = (
                     self.configuration.reserved_share_percentage)
+                pool['reserved_snapshot_percentage'] = (
+                    self.configuration.reserved_share_from_snapshot_percentage
+                    or self.configuration.reserved_share_percentage)
                 pool['dedupe'] = True
                 pool['compression'] = True
                 pool['thin_provisioning'] = True
