@@ -14,6 +14,7 @@
 #    under the License.
 
 import copy
+from oslo_serialization import jsonutils
 
 from manila.common import constants
 import manila.tests.share.drivers.netapp.fakes as na_fakes
@@ -27,6 +28,7 @@ DRIVER_NAME = 'fake_driver_name'
 APP_VERSION = 'fake_app_vsersion'
 HOST_NAME = 'fake_host'
 POOL_NAME = 'fake_pool'
+FLEXGROUP_STYLE_EXTENDED = 'flexgroup'
 POOL_NAME_2 = 'fake_pool_2'
 VSERVER1 = 'fake_vserver_1'
 VSERVER2 = 'fake_vserver_2'
@@ -36,6 +38,7 @@ VOLUME_NAME_TEMPLATE = 'share_%(share_id)s'
 VSERVER_NAME_TEMPLATE = 'os_%s'
 AGGREGATE_NAME_SEARCH_PATTERN = '(.*)'
 SHARE_NAME = 'share_7cf7c200_d3af_4e05_b87e_9167c95dfcad'
+SHARE_NAME2 = 'share_d24e7257_124e_4fb6_b05b_d384f660bc85'
 SHARE_INSTANCE_NAME = 'share_d24e7257_124e_4fb6_b05b_d384f660bc85'
 FLEXVOL_NAME = 'fake_volume'
 JUNCTION_PATH = '/%s' % FLEXVOL_NAME
@@ -57,6 +60,7 @@ CONSISTENCY_GROUP_ID2 = '35f5c1ea-45fb-40c4-98ae-2a2a17554159'
 CG_SNAPSHOT_ID = '6ddd8a6b-5df7-417b-a2ae-3f6e449f4eea'
 CG_SNAPSHOT_MEMBER_ID1 = '629f79ef-b27e-4596-9737-30f084e5ba29'
 CG_SNAPSHOT_MEMBER_ID2 = 'e876aa9c-a322-4391-bd88-9266178262be'
+SERVER_ID = 'd5e90724-6f28-4944-858a-553138bdbd29'
 FREE_CAPACITY = 10000000000
 TOTAL_CAPACITY = 20000000000
 AGGREGATE = 'manila_aggr_1'
@@ -70,6 +74,7 @@ NODE_DATA_PORT = 'e0c'
 NODE_DATA_PORTS = ('e0c', 'e0d')
 LIF_NAME_TEMPLATE = 'os_%(net_allocation_id)s'
 SHARE_TYPE_ID = '26e89a5b-960b-46bb-a8cf-0778e653098f'
+SHARE_TYPE_ID_2 = '2a06887e-25b5-486e-804a-d84c2d806feb'
 SHARE_TYPE_NAME = 'fake_share_type'
 IPSPACE = 'fake_ipspace'
 IPSPACE_ID = '27d38c27-3e8b-4d7d-9d91-fcf295e3ac8f'
@@ -81,10 +86,27 @@ MANILA_HOST_NAME_2 = '%(host)s@%(backend)s#%(pool)s' % {
     'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME_2}
 MANILA_HOST_NAME_3 = '%(host)s@%(backend)s#%(pool)s' % {
     'host': HOST_NAME, 'backend': BACKEND_NAME_2, 'pool': POOL_NAME_2}
+SERVER_HOST = '%(host)s@%(backend)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME}
+SERVER_HOST_2 = '%(host)s@%(backend)s' % {
+    'host': HOST_NAME, 'backend': BACKEND_NAME_2}
 QOS_EXTRA_SPEC = 'netapp:maxiops'
 QOS_SIZE_DEPENDENT_EXTRA_SPEC = 'netapp:maxbpspergib'
 QOS_NORMALIZED_SPEC = 'maxiops'
 QOS_POLICY_GROUP_NAME = 'fake_qos_policy_group_name'
+FPOLICY_POLICY_NAME = 'fake_fpolicy_name'
+FPOLICY_EVENT_NAME = 'fake_fpolicy_event_name'
+FPOLICY_PROTOCOL = 'cifs'
+FPOLICY_FILE_OPERATIONS = 'create,write,rename'
+FPOLICY_FILE_OPERATIONS_LIST = ['create', 'write', 'rename']
+FPOLICY_ENGINE = 'native'
+FPOLICY_EXT_TO_INCLUDE = 'avi'
+FPOLICY_EXT_TO_INCLUDE_LIST = ['avi']
+FPOLICY_EXT_TO_EXCLUDE = 'jpg,mp3'
+FPOLICY_EXT_TO_EXCLUDE_LIST = ['jpg', 'mp3']
+
+JOB_ID = '123'
+JOB_STATE = 'success'
 
 CLIENT_KWARGS = {
     'username': 'admin',
@@ -92,6 +114,7 @@ CLIENT_KWARGS = {
     'hostname': '127.0.0.1',
     'vserver': None,
     'transport_type': 'https',
+    'ssl_cert_path': '/etc/ssl/certs/',
     'password': 'pass',
     'port': '443',
     'api_trace_pattern': '(.*)',
@@ -114,6 +137,7 @@ SHARE = {
     'status': constants.STATUS_AVAILABLE,
     'share_server': None,
     'encrypt': False,
+    'share_id': SHARE_ID,
 }
 
 SHARE_INSTANCE = {
@@ -138,6 +162,7 @@ FLEXVOL_TO_MANAGE = {
     'type': 'rw',
     'style': 'flex',
     'size': '1610612736',  # rounds up to 2 GB
+    'style-extended': FLEXGROUP_STYLE_EXTENDED,
 }
 
 FLEXVOL_WITHOUT_QOS = copy.deepcopy(FLEXVOL_TO_MANAGE)
@@ -162,6 +187,15 @@ FLEXVOL = {
     'owning-vserver-name': VSERVER1,
 }
 
+SHARE_TYPE_EXTRA_SPEC = {
+    'snapshot_support': True,
+    'create_share_from_snapshot_support': True,
+    'revert_to_snapshot_support': True,
+    'mount_snapshot_support': False,
+    'driver_handles_share_servers': True,
+    'availability_zones': [],
+}
+
 EXTRA_SPEC = {
     'netapp:thin_provisioned': 'true',
     'netapp:snapshot_policy': 'default',
@@ -173,6 +207,76 @@ EXTRA_SPEC = {
     'netapp_disk_type': 'FCAL',
     'netapp_raid_type': 'raid4',
     'netapp_flexvol_encryption': 'true',
+    'netapp:tcp_max_xfer_size': 100,
+    'netapp:udp_max_xfer_size': 100,
+    'netapp:adaptive_qos_policy_group': None,
+}
+
+EXTRA_SPEC_WITH_REPLICATION = copy.copy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_REPLICATION.update({
+    'replication_type': 'dr'
+})
+
+EXTRA_SPEC_WITH_FPOLICY = copy.copy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_FPOLICY.update(
+    {'fpolicy_extensions_to_include': FPOLICY_EXT_TO_INCLUDE,
+     'fpolicy_extensions_to_exclude': FPOLICY_EXT_TO_EXCLUDE,
+     'fpolicy_file_operations': FPOLICY_FILE_OPERATIONS})
+
+NFS_CONFIG_DEFAULT = {
+    'tcp-max-xfer-size': 65536,
+    'udp-max-xfer-size': 32768,
+}
+
+NFS_CONFIG_TCP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:tcp_max_xfer_size': 100,
+    },
+    'expected': {
+        'tcp-max-xfer-size': 100,
+        'udp-max-xfer-size': NFS_CONFIG_DEFAULT['udp-max-xfer-size'],
+    }
+}
+
+NFS_CONFIG_UDP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:udp_max_xfer_size': 100,
+    },
+    'expected': {
+        'tcp-max-xfer-size': NFS_CONFIG_DEFAULT['tcp-max-xfer-size'],
+        'udp-max-xfer-size': 100,
+    }
+}
+
+NFS_CONFIG_TCP_UDP_MAX = {
+    'tcp-max-xfer-size': 100,
+    'udp-max-xfer-size': 100,
+}
+
+NFS_CONFIG_TCP_MAX = {
+    'tcp-max-xfer-size': 100,
+    'udp-max-xfer-size': NFS_CONFIG_DEFAULT['udp-max-xfer-size'],
+}
+
+NFS_CONFIG_UDP_MAX = {
+    'tcp-max-xfer-size': NFS_CONFIG_DEFAULT['tcp-max-xfer-size'],
+    'udp-max-xfer-size': 100,
+}
+
+NFS_CONFIG_TCP_UDP_MAX_DDT = {
+    'extra_specs': {
+        'netapp:tcp_max_xfer_size': 100,
+        'netapp:udp_max_xfer_size': 100,
+    },
+    'expected': NFS_CONFIG_TCP_UDP_MAX,
+}
+
+SHARE_GROUP_REF = {
+    'share_types': [
+        {'share_type_id': 'id1'},
+        {'share_type_id': 'id2'},
+        {'share_type_id': 'id3'},
+    ],
 }
 
 EXTRA_SPEC_WITH_QOS = copy.deepcopy(EXTRA_SPEC)
@@ -181,6 +285,12 @@ EXTRA_SPEC_WITH_QOS.update({
     QOS_EXTRA_SPEC: '3000',
 })
 
+EXTRA_SPEC_WITH_FPOLICY = copy.deepcopy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_FPOLICY.update(
+    {'netapp:fpolicy_extensions_to_include': FPOLICY_EXT_TO_INCLUDE,
+     'netapp:fpolicy_extensions_to_exclude': FPOLICY_EXT_TO_EXCLUDE,
+     'netapp:fpolicy_file_operations': FPOLICY_FILE_OPERATIONS})
+
 EXTRA_SPEC_WITH_SIZE_DEPENDENT_QOS = copy.deepcopy(EXTRA_SPEC)
 EXTRA_SPEC_WITH_SIZE_DEPENDENT_QOS.update({
     'qos': True,
@@ -197,12 +307,27 @@ PROVISIONING_OPTIONS = {
     'split': True,
     'encrypt': False,
     'hide_snapdir': False,
+    'adaptive_qos_policy_group': None,
 }
 
 PROVISIONING_OPTIONS_WITH_QOS = copy.deepcopy(PROVISIONING_OPTIONS)
 PROVISIONING_OPTIONS_WITH_QOS.update(
     {'qos_policy_group': QOS_POLICY_GROUP_NAME})
 
+PROVISIONING_OPTS_WITH_ADAPT_QOS = copy.deepcopy(PROVISIONING_OPTIONS)
+PROVISIONING_OPTS_WITH_ADAPT_QOS.update(
+    {'adaptive_qos_policy_group': QOS_POLICY_GROUP_NAME})
+
+PROVISIONING_OPTIONS_WITH_FPOLICY = copy.deepcopy(PROVISIONING_OPTIONS)
+PROVISIONING_OPTIONS_WITH_FPOLICY.update(
+    {'fpolicy_extensions_to_include': FPOLICY_EXT_TO_INCLUDE,
+     'fpolicy_extensions_to_exclude': FPOLICY_EXT_TO_EXCLUDE,
+     'fpolicy_file_operations': FPOLICY_FILE_OPERATIONS})
+
+PROVISIONING_OPTIONS_INVALID_FPOLICY = copy.deepcopy(PROVISIONING_OPTIONS)
+PROVISIONING_OPTIONS_INVALID_FPOLICY.update(
+    {'fpolicy_file_operations': FPOLICY_FILE_OPERATIONS})
+
 PROVISIONING_OPTIONS_BOOLEAN = {
     'thin_provisioned': True,
     'dedup_enabled': False,
@@ -226,18 +351,30 @@ PROVISIONING_OPTIONS_STRING = {
     'snapshot_policy': 'default',
     'language': 'en-US',
     'max_files': 5000,
+    'adaptive_qos_policy_group': None,
+    'fpolicy_extensions_to_exclude': None,
+    'fpolicy_extensions_to_include': None,
+    'fpolicy_file_operations': None,
 }
 
 PROVISIONING_OPTIONS_STRING_MISSING_SPECS = {
     'snapshot_policy': 'default',
     'language': 'en-US',
     'max_files': None,
+    'adaptive_qos_policy_group': None,
+    'fpolicy_extensions_to_exclude': None,
+    'fpolicy_extensions_to_include': None,
+    'fpolicy_file_operations': None,
 }
 
 PROVISIONING_OPTIONS_STRING_DEFAULT = {
     'snapshot_policy': None,
     'language': None,
     'max_files': None,
+    'adaptive_qos_policy_group': None,
+    'fpolicy_extensions_to_exclude': None,
+    'fpolicy_extensions_to_include': None,
+    'fpolicy_file_operations': None,
 }
 
 SHORT_BOOLEAN_EXTRA_SPEC = {
@@ -248,11 +385,13 @@ STRING_EXTRA_SPEC = {
     'netapp:snapshot_policy': 'default',
     'netapp:language': 'en-US',
     'netapp:max_files': 5000,
+    'netapp:adaptive_qos_policy_group': None,
 }
 
 SHORT_STRING_EXTRA_SPEC = {
     'netapp:snapshot_policy': 'default',
     'netapp:language': 'en-US',
+    'netapp:adaptive_qos_policy_group': None,
 }
 
 INVALID_EXTRA_SPEC = {
@@ -270,6 +409,14 @@ INVALID_MAX_FILE_EXTRA_SPEC = {
     'netapp:max_files': -1,
 }
 
+INVALID_TCP_MAX_XFER_SIZE_EXTRA_SPEC = {
+    'netapp:tcp_max_xfer_size': -1,
+}
+
+INVALID_UDP_MAX_XFER_SIZE_EXTRA_SPEC = {
+    'netapp:udp_max_xfer_size': -1,
+}
+
 EMPTY_EXTRA_SPEC = {}
 
 SHARE_TYPE = {
@@ -293,6 +440,14 @@ REMAPPED_OVERLAPPING_EXTRA_SPEC = {
     'netapp:thin_provisioned': 'false',
 }
 
+VOLUME_AUTOSIZE_ATTRS = {
+    'mode': 'off',
+    'grow-threshold-percent': '85',
+    'shrink-threshold-percent': '50',
+    'maximum-size': '1258288',
+    'minimum-size': '1048576',
+}
+
 USER_NETWORK_ALLOCATIONS = [
     {
         'id': '132dbb10-9a36-46f2-8d89-3d909830c356',
@@ -371,6 +526,11 @@ SHARE_SERVER = {
     },
     'network_allocations': (USER_NETWORK_ALLOCATIONS +
                             ADMIN_NETWORK_ALLOCATIONS),
+    'host': SERVER_HOST,
+    'share_network_subnet': {
+        'neutron_net_id': 'fake_neutron_net_id',
+        'neutron_subnet_id': 'fake_neutron_subnet_id'
+    }
 }
 
 SHARE_SERVER_2 = {
@@ -381,8 +541,78 @@ SHARE_SERVER_2 = {
     },
     'network_allocations': (USER_NETWORK_ALLOCATIONS +
                             ADMIN_NETWORK_ALLOCATIONS),
+    'host': SERVER_HOST_2,
+    'share_network_subnet': {
+        'neutron_net_id': 'fake_neutron_net_id_2',
+        'neutron_subnet_id': 'fake_neutron_subnet_id_2'
+    }
 }
 
+VSERVER_INFO = {
+    'name': 'fake_vserver_name',
+    'subtype': 'default',
+    'operational_state': 'running',
+    'state': 'running',
+}
+
+SHARE_SERVER_NFS_TCP = {
+    'id': 'fake_nfs_id_tcp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_TCP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_UDP = {
+    'id': 'fake_nfs_id_udp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_UDP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_TCP_UDP = {
+    'id': 'fake_nfs_id_tcp_udp',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_TCP_UDP_MAX),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NO_NFS_NONE = {
+    'id': 'fake_no_nfs_id_none',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NO_DETAILS = {
+    'id': 'id_no_datails',
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVER_NFS_DEFAULT = {
+    'id': 'fake_id_nfs_default',
+    'backend_details': {
+        'vserver_name': VSERVER2,
+        'nfs_config': jsonutils.dumps(NFS_CONFIG_DEFAULT),
+    },
+    'host': 'fake_host@fake_backend',
+}
+
+SHARE_SERVERS = [
+    SHARE_SERVER_NFS_TCP,
+    SHARE_SERVER_NFS_UDP,
+    SHARE_SERVER_NFS_TCP_UDP,
+    SHARE_SERVER_NFS_DEFAULT,
+    SHARE_SERVER_NO_NFS_NONE,
+    SHARE_SERVER_NO_DETAILS,
+]
+
 VSERVER_PEER = [{
     'vserver': VSERVER1,
     'peer-vserver': VSERVER2,
@@ -640,6 +870,46 @@ AGGREGATE_CAPACITIES = {
     }
 }
 
+FLEXGROUP_POOL_NAME = 'flexgroup_pool'
+
+FLEXGROUP_POOL_AGGR = [AGGREGATES[0], AGGREGATES[1]]
+
+FLEXGROUP_POOL_OPT = {
+    FLEXGROUP_POOL_NAME: FLEXGROUP_POOL_AGGR,
+}
+
+FLEXGROUP_POOL_OPT_RAW = {
+    FLEXGROUP_POOL_NAME: '%s %s' % (AGGREGATES[0], AGGREGATES[1]),
+}
+
+FLEXGROUP_POOL = {
+    'pool_name': FLEXGROUP_POOL_NAME,
+    'netapp_aggregate': '%s %s' % (AGGREGATES[0], AGGREGATES[1]),
+    'total_capacity_gb': 6.6,
+    'free_capacity_gb': 2.2,
+    'allocated_capacity_gb': 4.39,
+    'reserved_percentage': 5,
+    'max_over_subscription_ratio': 2.0,
+    'dedupe': [True, False],
+    'compression': [True, False],
+    'thin_provisioning': [True, False],
+    'netapp_flexvol_encryption': True,
+    'netapp_raid_type': 'raid4 raid_dp',
+    'netapp_disk_type': ['FCAL', 'SATA', 'SSD'],
+    'netapp_hybrid_aggregate': 'false true',
+    'utilization': 50.0,
+    'filter_function': FLEXGROUP_POOL_NAME,
+    'goodness_function': 'goodness',
+    'snapshot_support': True,
+    'create_share_from_snapshot_support': True,
+    'revert_to_snapshot_support': True,
+    'qos': True,
+    'security_service_update_support': True,
+    'netapp_flexgroup': True,
+}
+
+FLEXGROUP_AGGR_SET = set(FLEXGROUP_POOL_OPT[FLEXGROUP_POOL_NAME])
+
 AGGREGATE_CAPACITIES_VSERVER_CREDS = {
     AGGREGATES[0]: {
         'available': 1181116007,  # 1.1 GB
@@ -655,21 +925,38 @@ SSC_INFO = {
         'netapp_disk_type': 'FCAL',
         'netapp_hybrid_aggregate': 'false',
         'netapp_aggregate': AGGREGATES[0],
+        'netapp_flexgroup': False,
     },
     AGGREGATES[1]: {
         'netapp_raid_type': 'raid_dp',
         'netapp_disk_type': ['SATA', 'SSD'],
         'netapp_hybrid_aggregate': 'true',
         'netapp_aggregate': AGGREGATES[1],
+        'netapp_flexgroup': False,
+    }
+}
+
+SSC_INFO_MAP = {
+    AGGREGATES[0]: {
+        'netapp_raid_type': 'raid4',
+        'netapp_disk_type': ['FCAL'],
+        'netapp_hybrid_aggregate': 'false',
+    },
+    AGGREGATES[1]: {
+        'netapp_raid_type': 'raid_dp',
+        'netapp_disk_type': ['SATA', 'SSD'],
+        'netapp_hybrid_aggregate': 'true',
     }
 }
 
 SSC_INFO_VSERVER_CREDS = {
     AGGREGATES[0]: {
         'netapp_aggregate': AGGREGATES[0],
+        'netapp_flexgroup': False,
     },
     AGGREGATES[1]: {
         'netapp_aggregate': AGGREGATES[1],
+        'netapp_flexgroup': False,
     }
 }
 
@@ -681,6 +968,8 @@ POOLS = [
         'free_capacity_gb': 1.1,
         'allocated_capacity_gb': 2.2,
         'reserved_percentage': 5,
+        'reserved_snapshot_percentage': 2,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -695,6 +984,8 @@ POOLS = [
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
         'qos': True,
+        'security_service_update_support': True,
+        'netapp_flexgroup': False,
     },
     {
         'pool_name': AGGREGATES[1],
@@ -703,6 +994,8 @@ POOLS = [
         'free_capacity_gb': 2.0,
         'allocated_capacity_gb': 4.0,
         'reserved_percentage': 5,
+        'reserved_snapshot_percentage': 2,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
@@ -717,28 +1010,34 @@ POOLS = [
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
         'qos': True,
+        'security_service_update_support': True,
+        'netapp_flexgroup': False,
     },
 ]
 
 POOLS_VSERVER_CREDS = [
     {
         'pool_name': AGGREGATES[0],
+        'filter_function': None,
+        'goodness_function': None,
         'netapp_aggregate': AGGREGATES[0],
         'total_capacity_gb': 'unknown',
         'free_capacity_gb': 1.1,
         'allocated_capacity_gb': 0.0,
         'reserved_percentage': 5,
+        'reserved_snapshot_percentage': 2,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
         'netapp_flexvol_encryption': True,
         'utilization': 50.0,
-        'filter_function': None,
-        'goodness_function': None,
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
         'qos': False,
+        'security_service_update_support': True,
+        'netapp_flexgroup': False,
     },
     {
         'pool_name': AGGREGATES[1],
@@ -747,17 +1046,19 @@ POOLS_VSERVER_CREDS = [
         'free_capacity_gb': 2.0,
         'allocated_capacity_gb': 0.0,
         'reserved_percentage': 5,
+        'reserved_snapshot_percentage': 2,
+        'max_over_subscription_ratio': 2.0,
         'dedupe': [True, False],
         'compression': [True, False],
         'thin_provisioning': [True, False],
         'netapp_flexvol_encryption': True,
         'utilization': 50.0,
-        'filter_function': None,
-        'goodness_function': None,
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
         'qos': False,
+        'security_service_update_support': True,
+        'netapp_flexgroup': False,
     },
 ]
 
@@ -1201,6 +1502,11 @@ PROCESSOR_INSTANCE_UUIDS = [
 ]
 PROCESSOR_INSTANCE_NAMES = ['processor0', 'processor1']
 
+SERVER_METADATA = {
+    'share_type_id': 'fake_id',
+    'host': 'fake_host',
+}
+
 PROCESSOR_COUNTERS = [
     {
         'node-name': 'cluster1-01',
@@ -1313,15 +1619,165 @@ EXPANDED_PROCESSOR_COUNTERS = [
     },
 ]
 
+SERVER_MIGRATION_CHECK_NOT_COMPATIBLE = {
+    'compatible': False,
+    'writable': None,
+    'nondisruptive': None,
+    'preserve_snapshots': None,
+    'migration_cancel': None,
+    'migration_get_progress': None,
+    'share_network_id': None,
+}
+
+CIFS_SECURITY_SERVICE = {
+    'id': 'fake_id',
+    'type': 'active_directory',
+    'password': 'fake_password',
+    'user': 'fake_user',
+    'ou': 'fake_ou',
+    'domain': 'fake_domain',
+    'dns_ip': 'fake_dns_ip',
+    'server': 'fake_server',
+}
+
+CIFS_SECURITY_SERVICE_2 = {
+    'id': 'fake_id_2',
+    'type': 'active_directory',
+    'password': 'fake_password_2',
+    'user': 'fake_user_2',
+    'ou': 'fake_ou_2',
+    'domain': 'fake_domain_2',
+    'dns_ip': 'fake_dns_ip_2',
+    'server': 'fake_server_2',
+}
+LDAP_LINUX_SECURITY_SERVICE = {
+    'id': 'fake_id',
+    'type': 'ldap',
+    'user': 'fake_user',
+    'password': 'fake_password',
+    'server': 'fake_server',
+    'ou': 'fake_ou',
+    'dns_ip': None,
+    'domain': None
+}
+
+
+LDAP_AD_SECURITY_SERVICE = {
+    'id': 'fake_id',
+    'type': 'ldap',
+    'user': 'fake_user',
+    'password': 'fake_password',
+    'domain': 'fake_domain',
+    'ou': 'fake_ou',
+    'dns_ip': 'fake_dns_ip',
+    'server': None,
+}
+
+KERBEROS_SECURITY_SERVICE = {
+    'id': 'fake_id_3',
+    'type': 'kerberos',
+    'password': 'fake_password_3',
+    'user': 'fake_user_3',
+    'domain': 'fake_realm',
+    'dns_ip': 'fake_dns_ip_3',
+    'server': 'fake_server_3',
+}
+
+KERBEROS_SECURITY_SERVICE_2 = {
+    'id': 'fake_id_4',
+    'type': 'kerberos',
+    'password': 'fake_password_4',
+    'user': 'fake_user_4',
+    'domain': 'fake_realm_2',
+    'dns_ip': 'fake_dns_ip_4',
+    'server': 'fake_server_4',
+}
+
+SHARE_NETWORK_SUBNET = {
+    'id': 'fake_share_net_subnet_d',
+    'neutron_subnet_id': '34950f50-a142-4328-8026-418ad4410b09',
+    'neutron_net_id': 'fa202676-531a-4446-bc0c-bcec15a72e82',
+    'network_type': 'fake_network_type',
+    'segmentation_id': 1234,
+    'ip_version': 4,
+    'cidr': 'fake_cidr',
+    'gateway': 'fake_gateway',
+    'mtu': 1509,
+}
+
+SHARE_NETWORK = {
+    'id': 'fake_share_net_id',
+    'project_id': 'fake_project_id',
+    'status': 'fake_status',
+    'name': 'fake_name',
+    'description': 'fake_description',
+    'security_services': [CIFS_SECURITY_SERVICE],
+    'subnets': [SHARE_NETWORK_SUBNET],
+}
+
+SHARE_TYPE_2 = copy.deepcopy(SHARE_TYPE)
+SHARE_TYPE_2['id'] = SHARE_TYPE_ID_2
+SHARE_TYPE_2['extra_specs'].update(SHARE_TYPE_EXTRA_SPEC)
+
+SHARE_REQ_SPEC = {
+    'share_properties': {
+        'size': SHARE['size'],
+        'project_id': SHARE['project_id'],
+        'snapshot_support': SHARE_TYPE_EXTRA_SPEC['snapshot_support'],
+        'create_share_from_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['create_share_from_snapshot_support'],
+        'revert_to_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['revert_to_snapshot_support'],
+        'mount_snapshot_support':
+            SHARE_TYPE_EXTRA_SPEC['mount_snapshot_support'],
+        'share_proto': SHARE['share_proto'],
+        'share_type_id': SHARE_TYPE_2['id'],
+        'is_public': True,
+        'share_group_id': None,
+        'source_share_group_snapshot_member_id': None,
+        'snapshot_id': None,
+    },
+    'share_instance_properties': {
+        'availability_zone_id': 'fake_az_1',
+        'share_network_id': SHARE_NETWORK['id'],
+        'share_server_id': SHARE_SERVER['id'],
+        'share_id': SHARE_ID,
+        'host': SHARE_INSTANCE['host'],
+        'status': SHARE_INSTANCE['status'],
+    },
+    'share_type': SHARE_TYPE_2,
+    'share_id': SHARE_ID,
+}
+
+SERVER_MIGRATION_REQUEST_SPEC = {
+    'shares_size': 10,
+    'snapshots_size': 10,
+    'shares_req_spec': [SHARE_REQ_SPEC],
+}
+
+CLIENT_GET_VOLUME_RESPONSE = {
+    'aggregate': AGGREGATE,
+    'junction-path': '/%s' % SHARE_NAME,
+    'name': SHARE_NAME,
+    'type': 'rw',
+    'style': 'flex',
+    'size': SHARE_SIZE,
+    'owning-vserver-name': VSERVER1,
+    'qos-policy-group-name': QOS_POLICY_GROUP_NAME,
+}
+
 
 def get_config_cmode():
     config = na_fakes.create_configuration_cmode()
     config.local_conf.set_override('share_backend_name', BACKEND_NAME)
     config.reserved_share_percentage = 5
+    config.reserved_share_from_snapshot_percentage = 2
+    config.max_over_subscription_ratio = 2.0
     config.netapp_login = CLIENT_KWARGS['username']
     config.netapp_password = CLIENT_KWARGS['password']
     config.netapp_server_hostname = CLIENT_KWARGS['hostname']
     config.netapp_transport_type = CLIENT_KWARGS['transport_type']
+    config.netapp_ssl_cert_path = CLIENT_KWARGS['ssl_cert_path']
     config.netapp_server_port = CLIENT_KWARGS['port']
     config.netapp_volume_name_template = VOLUME_NAME_TEMPLATE
     config.netapp_aggregate_name_search_pattern = AGGREGATE_NAME_SEARCH_PATTERN
@@ -1340,3 +1796,7 @@ def get_network_info(user_network_allocation, admin_network_allocation):
     net_info['admin_network_allocations'] = admin_network_allocation
 
     return net_info
+
+
+def fake_get_filter_function(pool=None):
+    return pool if pool else 'filter'
